#include "monWidgetGL.hpp"

#include "libmesh/mesh.hpp"
#include "libmesh/mesh_io.hpp"
#include "libmesh/opengl_drawer.hpp"
#include "lib3d/matrix3.hpp"

#include <GL/glu.h>


#include <iostream>

using cpe::vec3;
using cpe::vec2;
using cpe::matrix3;
using cpe::mesh;

monWidgetGL::monWidgetGL(QGLWidget *parent) :
    QGLWidget(parent)
{
    QWidget::setFocusPolicy(Qt::StrongFocus);
    etat_affichage=true;

    translation=vec3(0,0,0);
}

monWidgetGL::~monWidgetGL()
{}

void monWidgetGL::initializeGL()
{
    std::cout<<"appel monWidgetGL::initializeGL()"<<std::endl;

    glClearColor(1.0, 1.0, 1.0, 0.0);

    //chargement de maillage
    mesh_0.load_file("data/camel.off");

    //rotation et translation du maillage
    mesh_0.apply(cpe::matrix3::rotation(vec3(1,0,0),-M_PI/2.0));
    mesh_0.apply(cpe::matrix3::rotation(vec3(0,1,0),3*M_PI/4.0));
    mesh_0.auto_scale(1.0);
    mesh_0+=vec3(0,0.8,-7);

    //calcul des normales par sommets
    mesh_0.compute_normal();




    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);

    //light
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glFrontFace( GL_CW ) ;
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    float light_position[4]={1.0,-8.0,-5.0,float(1.0)};
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glEnable(GL_COLOR_MATERIAL);

}

void monWidgetGL::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);



    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60,1.0,0.1,50.0);
    glRotated(180.0*angle/M_PI,0,1,0);
    glTranslated(translation.x(),translation.y(),translation.z());


    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();




    if(etat_affichage==true)
    {
        glColor3f(0.8,0.7,0.6);
        cpe::opengl_drawer::draw(mesh_0);


        affiche_quadrangle_sol();
    }


}



void monWidgetGL::afficher()
{
    etat_affichage=!etat_affichage;

    updateGL();
}
void monWidgetGL::wireframe(bool est_actif)
{
    if(est_actif==true)
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);

    updateGL();
}

void monWidgetGL::keyPressEvent(QKeyEvent *event)
{

    double d_angle=0.05;
    double d_radial=0.1;
    double d_height=0.1;

    vec3 d_tr(-d_radial*sin(angle),0,d_radial*cos(angle));
    int current=event->key();


    switch(current)
    {
    case Qt::Key_Up:
        translation += d_tr;
        break;
    case Qt::Key_Down:
        translation -= d_tr;
        break;
    case Qt::Key_Left:
        angle-=d_angle;
        break;
    case Qt::Key_Right:
        angle+=d_angle;
        break;

    case Qt::Key_A:
        translation.y() -= d_height;
        break;
    case Qt::Key_Q:
        translation.y() += d_height;
        break;
    }

    updateGL();
}

void monWidgetGL::affiche_quadrangle_sol()
{
    cpe::vec3 n(0,-1,0);
    cpe::vec3 c(0.9,0.9,0.9);
    cpe::vec3 p[4]={vec3(-8,-0.2,-8),
               vec3( 8,-0.2,-8),
               vec3( 8,-0.2, 8),
               vec3( 8,-0.2, 8)};
    unsigned int connectivity[]={0,1,2 ,
                                 0,2,3};

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    //setup pointers
    glVertexPointer(3,GL_FLOAT,0,p[0].pointer());
    glColorPointer(3,GL_FLOAT,0,c.pointer());
    glNormalPointer(GL_FLOAT,0,n.pointer());

    //draw triangles
    glDrawElements(GL_TRIANGLES,3*2,GL_UNSIGNED_INT,connectivity);

    //disable client state
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_COLOR_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);


/*
    glNormal3f(0,-1,0);
    glColor3f(0.9,0.9,0.9);
    glBegin(GL_QUADS);
    glVertex3f(-8,-0.2,-8);
    glVertex3f( 8,-0.2,-8);
    glVertex3f( 8,-0.2, 8);
    glVertex3f(-8,-0.2, 8);
    glEnd();*/
}

